==> modul8-0.12.0.tgz: package/examples/advanced/output.js <==
(function(){
window.QQ = {data:{}};
QQ.data.test = "{\"hi\": \"there\"}\n";
(function(){

/**
 * modul8 v0.12.0
 */

var config    = {"namespace":"QQ","domains":["app","shared"],"arbiters":{"monolith":["monolith"]},"logging":1}
  , ns        = window[config.namespace]
  , domains   = config.domains
  , arbiters  = []
  , exports   = {}
  , DomReg    = /^(.*)::/;

/**
 * Initialize the exports container with domain names + move data to it
 */
for (var i = 0; i < domains.length; i++)
  exports[domains[i]] = {};

exports.M8 = {};
exports.external = {};
exports.data = ns.data;
delete ns.data;

/**
 * Attach arbiters to the require system then delete them from the global scope
 */
for (var name in config.arbiters) {
  var arry  = config.arbiters[name]
    , temp = window[name];
  arbiters.push(name);
  for (var j = 0; j < arry.length; j++) delete window[arry[j]];
  exports.M8[name] = temp;
}

/**
 * Converts a relative path to an absolute one
 */
function toAbsPath(pathName, relReqStr) {
  var folders = pathName.split('/').slice(0, -1);
  while (relReqStr.slice(0, 3) === '../') {
    folders = folders.slice(0, -1);
    relReqStr = relReqStr.slice(3);
  }
  return folders.concat(relReqStr.split('/')).join('/');
};

/**
 * Require Factory for ns.define
 * Each (domain,path) gets a specialized require function from this
 */
function makeRequire(dom, pathName) {
  return function(reqStr) {
    var o, scannable, k;

    if (config.logging >= 4)
      console.debug('modul8: '+dom+':'+pathName+" <- "+reqStr);

    if (reqStr.slice(0, 2) === './') {
      scannable = [dom];
      reqStr = toAbsPath(pathName, reqStr.slice(2));
    } else if (reqStr.slice(0,3) === '../') {
      scannable = [dom];
      reqStr = toAbsPath(pathName, reqStr);
    } else if (DomReg.test(reqStr)) {
      scannable = [reqStr.match(DomReg)[1]];
      reqStr = reqStr.split('::')[1];
    } else if (arbiters.indexOf(reqStr) >= 0) {
      scannable = ['M8'];
    } else {
      scannable = [dom].concat(domains.filter(function(e) {return e !== dom;}));
    }
    reqStr = reqStr.split('.')[0];
    if (reqStr.slice(-1) === '/') {
      reqStr += 'index';
      var skipFolder = true;
    }

    if (config.logging >= 3)
      console.log('modul8: '+dom+':'+pathName+" <- "+reqStr);
    if (config.logging >= 4)
      console.debug('modul8: scanned '+JSON.stringify(scannable))

    for (k = 0; k < scannable.length; k++) {
      o = scannable[k];
      if (exports[o][reqStr])
        return exports[o][reqStr];

      if (!skipFolder && exports[o][reqStr + '/index'])
        return exports[o][reqStr + '/index'];
    }

    if (config.logging >= 1)
      console.error("modul8: Unable to resolve require for: " + reqStr);
  };
};

ns.define = function(name, domain, fn) {
  var module = {};
  fn(makeRequire(domain, name), module, exports[domain][name] = {});
  if (module.exports) {
    delete exports[domain][name];
    exports[domain][name] = module.exports;
  }
};

/**
 * Public Debug API
 */

ns.inspect = function(domain) {
  console.log(exports[domain]);
};

ns.domains = function() {
  return domains.concat(['external','data']);
};

ns.require = makeRequire('app', 'CONSOLE');

/**
 * Live Extension API
 */

ns.data = function(name, exported) {
  if (exports.data[name]) delete exports.data[name];
  if (exported) exports.data[name] = exported;
};

ns.external = function(name, exported) {
  if (exports.external[name]) delete exports.external[name];
  if (exported) exports.external[name] = exported;
};

})();

// shared code

QQ.define('calc','shared',function(require, module, exports){

module.exports = {
  divides: function(d, n) {
    return !(d % n);
  }
};

});
QQ.define('validation','shared',function(require, module, exports){
var divides;

divides = require('./calc').divides;

exports.isLeapYear = function(yr) {
  return divides(yr, 4) && (!divides(yr, 100) || divides(yr, 400));
};

});

// app code - safety wrap


(function(){
QQ.define('bigthing/sub2','app',function(require, module, exports){

module.exports = function(str) {
  return console.log(str);
};

});
QQ.define('helper','app',function(require, module, exports){
var testRunner;

module.exports = function(str) {
  return console.log(str);
};

});
QQ.define('bigthing/sub1','app',function(require, module, exports){
var sub2;

sub2 = require('./sub2');

exports.doComplex = function(str) {
  return sub2(str + ' (sub1 added this, passing to sub2)');
};

});
QQ.define('main','app',function(require, module, exports){
var b, helper, m, test, v;

helper = require('./helper');

helper('hello from app via helper');

b = require('bigthing/sub1');

b.doComplex('app calls up to sub1');

v = require('validation.coffee');

console.log('2004 isLeapYear?', v.isLeapYear(2004));

m = require('monolith');

console.log("monolith:" + m);

test = require('data::test');

console.log('injected data:', test);

});
})();
})();

==> modul8-0.13.1.tgz: package/examples/advanced/output.js <==
(function(){
window.QQ = {data:{}};
QQ.data.test = {"hi": "there"}
;
(function(){
/**
 * modul8 v0.12.0
 */

var config    = {"namespace":"QQ","domains":["app","shared"],"arbiters":{"monolith":["monolith"]},"logging":1}
  , ns        = window[config.namespace]
  , domains   = config.domains
  , arbiters  = []
  , exports   = {}
  , DomReg    = /^([\w]*)::/;

/**
 * Initialize the exports container with domain names + move data to it
 */
exports.M8 = {};
exports.external = {};
exports.data = ns.data;
delete ns.data;

domains.forEach(function(e){
  exports[e] = {};
});

/**
 * Attach arbiters to the require system then delete them from the global scope
 */
Object.keys(config.arbiters).forEach(function(name){
  var arbAry = config.arbiters[name];
  arbiters.push(name);
  exports.M8[name] = window[arbAry[0]];
  arbAry.forEach(function(e){
    delete window[e];
  });
});

/**
 * Converts a relative path to an absolute one
 */
function toAbsPath(pathName, relReqStr) {
  var folders = pathName.split('/').slice(0, -1);
  while (relReqStr.slice(0, 3) === '../') {
    folders = folders.slice(0, -1);
    relReqStr = relReqStr.slice(3);
  }
  return folders.concat(relReqStr.split('/')).join('/');
}

/**
 * Require Factory for ns.define
 * Each (domain,path) gets a specialized require function from this
 */
function makeRequire(dom, pathName) {
  return function(reqStr) {
    var o, scannable, k, skipFolder;

    if (config.logging >= 4) {
      console.debug('modul8: '+dom+':'+pathName+" <- "+reqStr);
    }

    if (reqStr.slice(0, 2) === './') {
      scannable = [dom];
      reqStr = toAbsPath(pathName, reqStr.slice(2));
    }
    else if (reqStr.slice(0,3) === '../') {
      scannable = [dom];
      reqStr = toAbsPath(pathName, reqStr);
    }
    else if (DomReg.test(reqStr)) {
      scannable = [reqStr.match(DomReg)[1]];
      reqStr = reqStr.split('::')[1];
    }
    else if (arbiters.indexOf(reqStr) >= 0) {
      scannable = ['M8'];
    }
    else {
      scannable = [dom].concat(domains.filter(function(e) {return e !== dom;}));
    }

    reqStr = reqStr.split('.')[0];
    if (reqStr.slice(-1) === '/') {
      reqStr += 'index';
      skipFolder = true;
    }

    if (config.logging >= 3) {
      console.log('modul8: '+dom+':'+pathName+' <- '+reqStr);
    }
    if (config.logging >= 4) {
      console.debug('modul8: scanned '+JSON.stringify(scannable));
    }

    for (k = 0; k < scannable.length; k += 1) {
      o = scannable[k];
      if (exports[o][reqStr]) {
        return exports[o][reqStr];
      }
      if (!skipFolder && exports[o][reqStr + '/index']) {
        return exports[o][reqStr + '/index'];
      }
    }

    if (config.logging >= 1) {
      console.error("modul8: Unable to resolve require for: " + reqStr);
    }
  };
}

ns.define = function(name, domain, fn) {
  var module = {};
  fn(makeRequire(domain, name), module, exports[domain][name] = {});
  if (module.exports) {
    delete exports[domain][name];
    exports[domain][name] = module.exports;
  }
};

/**
 * Public Debug API
 */

ns.inspect = function(domain) {
  console.log(exports[domain]);
};

ns.domains = function() {
  return domains.concat(['external','data']);
};

ns.require = makeRequire('app', 'CONSOLE');

/**
 * Live Extension API
 */

ns.data = function(name, exported) {
  if (exports.data[name]) {
    delete exports.data[name];
  }
  if (exported) {
    exports.data[name] = exported;
  }
};

ns.external = function(name, exported) {
  if (exports.external[name]) {
    delete exports.external[name];
  }
  if (exported) {
    exports.external[name] = exported;
  }
};

})();

// shared code

QQ.define('calc','shared',function(require, module, exports){

module.exports = {
  divides: function(d, n) {
    return !(d % n);
  }
};

});
QQ.define('validation','shared',function(require, module, exports){
var divides;

divides = require('./calc').divides;

exports.isLeapYear = function(yr) {
  return divides(yr, 4) && (!divides(yr, 100) || divides(yr, 400));
};

});

// app code - safety wrap


(function(){
QQ.define('bigthing/sub2','app',function(require, module, exports){

module.exports = function(str) {
  return console.log(str);
};

});
QQ.define('helper','app',function(require, module, exports){
var testRunner;

module.exports = function(str) {
  return console.log(str);
};

});
QQ.define('bigthing/sub1','app',function(require, module, exports){
var sub2;

sub2 = require('./sub2');

exports.doComplex = function(str) {
  return sub2(str + ' (sub1 added this, passing to sub2)');
};

});
QQ.define('main','app',function(require, module, exports){
var b, helper, m, test, v;

helper = require('./helper');

helper('hello from app via helper');

b = require('bigthing/sub1');

b.doComplex('app calls up to sub1');

v = require('validation.coffee');

console.log('2004 isLeapYear?', v.isLeapYear(2004));

m = require('monolith');

console.log("monolith:" + m);

test = require('data::test');

console.log('injected data:', test);

});
})();
})();
